import os

from cmutils.config_loaders import IniConfigLoader
from peewee import *
import json
from playhouse.shortcuts import ReconnectMixin

store_database_proxy = Proxy()


class ReconnectMySQLDatabase(ReconnectMixin, MySQLDatabase):
  pass

def init_store(host, user, password, database):
  store_db_connection = ReconnectMySQLDatabase(
    host=host,
    user=user,
    password=password,
    database=database
  )
  store_database_proxy.initialize(store_db_connection)


class StoreBaseModel(Model):
  def __str__(self):
    r = {}
    for k in self._data.keys():
      try:
        r[k] = str(getattr(self, k))
      except:
        r[k] = json.dumps(getattr(self, k))

    return str(r)

  class Meta:
    database = store_database_proxy


class Store(StoreBaseModel):
  id = IntegerField()
  name = CharField()
  code = CharField()
  platfrom = CharField()
  api_credential = CharField()
  url = CharField()

  class Meta:
    db_table = 'stores'


class StoreSeller(StoreBaseModel):
  id = IntegerField()
  name = CharField()
  slug = CharField()
  state = CharField()
  email = CharField()
  code = CharField()

  class Meta:
    db_table = 'store_sellers'


class StoreProduct(StoreBaseModel):
  id = IntegerField()
  store_seller_id = IntegerField()
  s_product_id = IntegerField()
  source = CharField()
  source_product_id = CharField()
  handle = CharField()

  class Meta:
    db_table = 'store_products'


class GmcStore(StoreBaseModel):
  id = IntegerField()
  merchant_id = CharField()
  service_account_data = CharField()
  active = BooleanField()

  class Meta:
    db_table = 'gmc_stores'


class GmcProduct(StoreBaseModel):
  id = IntegerField()
  gmc_store_id = IntegerField()
  gmc_id = CharField()
  offer_id = CharField()
  expiration_date = DateTimeField()
  price = FloatField()
  currency = CharField()
  availability = CharField()
  seller_id = CharField()
  store_product_id = IntegerField()
  handle = CharField()

  class Meta:
    db_table = 'gmc_products'


class GmcProductStatus(StoreBaseModel):
  id = IntegerField()
  gmc_product_id = IntegerField()
  item_disapproved = BooleanField()
  country = CharField()
  issues = CharField()

  class Meta:
    db_table = 'gmc_product_statuses'


class GmcIssue(StoreBaseModel):
  id = IntegerField()
  code = CharField()
  servability = CharField()
  resolution = CharField()
  detail = CharField()
  documentation = CharField()

  class Meta:
    db_table = 'gmc_issues'


class GoogleAdsAccount(StoreBaseModel):
  id = IntegerField()
  impersonated_email = CharField()
  customer_id = CharField()
  service_account_data = CharField()
  active = BooleanField()

  class Meta:
    db_table = 'google_ads_accounts'


class GoogleAdsPerformance(StoreBaseModel):
  id = IntegerField()
  google_ads_account_id = IntegerField()
  merchant_id = CharField()
  item_id = CharField()
  impressions = CharField()
  clicks = IntegerField()
  ctr = FloatField()
  average_cpc = FloatField()
  orders = IntegerField()
  units_sold = IntegerField()
  revenue = FloatField()
  cost = FloatField()
  average_order_value = FloatField()
  conversions = IntegerField()
  conversions_value = FloatField()
  cost_per_conversion = FloatField()
  value_per_conversion = FloatField()
  roas = FloatField()
  all_conversions = IntegerField()
  all_conversions_value = FloatField()
  cost_per_all_conversions = FloatField()
  value_per_all_conversions = FloatField()
  search_impression_share = FloatField()
  search_click_share = FloatField()
  type = IntegerField()
  last_update_date = DateField()

  class Meta:
    db_table = 'google_ads_performances'
