import datetime

import dateutil
import dateutil.parser

from em_tasks import logger
from em_tasks.shopify.product_util import ProductUtil
from em_tasks.store.store_util import StoreUtil


class SpapiUpdateItemOffersTask():
  def __init__(self, spapi, offer_service, marketplace, asins, condition='new', ttl=24, force=False):
    self.spapi = spapi
    self.offer_service = offer_service
    self.marketplace = marketplace.lower()
    self.asins = asins
    self.condition = condition.lower()
    self.ttl = ttl
    self.force = force

  def run(self):
    offer_type = 'lowest_offer_listings'
    now = datetime.datetime.now()
    offer_expire_time = now - datetime.timedelta(hours=self.ttl)
    asins_without_offer = None
    if self.force:
      asins_without_offer = list(self.asins)
    else:
      offers = self.offer_service.search_offers(
        offer_type, self.asins, self.marketplace, self.condition)
      asins_without_offer = dict()
      for asin in self.asins:
        if asin not in offers or not offers['asin']:
          asins_without_offer[asin] = None
          continue

        offer = offers[asin]
        offer_time_s = offer.get('time', None)
        if not offer_time_s:
          asins_without_offer[asin] = None
          continue

        try:
          offer_time = dateutil.parser.parse(offer_time_s)

          # Offer expired
          if offer_time < offer_expire_time:
            asins_without_offer[asin] = None
            continue
        except Exception as e:
          asins_without_offer[asin] = None

      asins_without_offer = list(asins_without_offer.keys())

    d = dict()
    chunks = [asins_without_offer[x:x + 20] for x in range(0, len(self.asins), 20)]
    for chunk in chunks:
      offers = None
      try:
        offers = self.spapi.get_item_offers_batch(self.marketplace, chunk, self.condition)
        logger.info('[SpapiOffer] %s', offers)
      except Exception as e:
        logger.exception(e)

      if not offers:
        return

      try:
        self.offer_service.save_item_offers(
          offer_type, offers, self.marketplace, self.condition)
        logger.info('[OfferSaved] %s', offers)
      except Exception as e:
        logger.warning(
          '[SaveOfferError] Marketplace: %s, Condition: %s, ASINs: %s',
          self.marketplace, self.condition, self.asins)
        logger.exception(e)
