import datetime

import dateutil
import dateutil.parser

from dropshipping.utils.offer_services import EsOfferService


class SpapiOfferService():
  def __init__(self, host, port, username=None, password=None):
    self.offer_service = EsOfferService(host, port, user, password)
    self.offer_type = 'lowest_offer_listings'

  def get_asins_without_offer(self, asins, marketplace, condition='new', ttl=24):
    now = datetime.datetime.now()
    offer_expire_time = now - datetime.timedelta(hours=self.ttl)

    offers = self.offer_service.search_offers(
      self.offer_type, asins, marketplace, condition)
    asins_without_offer = dict()
    for asin in self.asins:
      if asin not in offers or not offers['asin']:
        asins_without_offer[asin] = None
        continue

      offer = offers[asin]
      offer_time_s = offer.get('time', None)
      if not offer_time_s:
        asins_without_offer[asin] = None
        continue

      try:
        offer_time = dateutil.parser.parse(offer_time_s)

        # Offer expired
        if offer_time < offer_expire_time:
          asins_without_offer[asin] = None
          continue
      except Exception as e:
        asins_without_offer[asin] = None

    return list(asins_without_offer.keys())

  def search_offers(self, asins, marketplace, condition='new'):
    return self.offer_service.search_offers(
      self.offer_type, asins, marketplace.lower(), condition)

  def save_offers(self, offers, marketplace, condition):
    self.offer_service.save_item_offers(
      self.offer_type, offers, self.marketplace, self.condition)
